use strict;
use Test::More 0.98;

use_ok $_ for qw/
    Test::JSON::RPC::Autodoc
    Test::JSON::RPC::Autodoc::Request
    Test::JSON::RPC::Autodoc::Response
    Test::JSON::RPC::Autodoc::Validator
/;

done_testing;

